=== modified file 'NEWS'
--- NEWS	2008-06-03 07:22:42 +0000
+++ NEWS	2009-03-24 04:49:51 +0000
@@ -4,7 +4,6 @@
 
 .. contents::
 
-
 bzr 1.6beta1 2008-06-02
 -----------------------
 

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2008-06-02 22:27:34 +0000
+++ bzrlib/branch.py	2009-03-24 04:49:51 +0000
@@ -919,7 +919,7 @@
             for (filename, content) in utf8_files:
                 branch_transport.put_bytes(
                     filename, content,
-                    mode=control_files._file_mode)
+                    mode=a_bzrdir._get_file_mode())
         finally:
             control_files.unlock()
         return self.open(a_bzrdir, _found=True)
@@ -1413,7 +1413,7 @@
         It is intended to be called by BzrBranch5.set_revision_history."""
         self._transport.put_bytes(
             'revision-history', '\n'.join(history),
-            mode=self.control_files._file_mode)
+            mode=self.bzrdir._get_file_mode())
 
     @needs_write_lock
     def set_revision_history(self, rev_history):
@@ -1735,7 +1735,7 @@
             self._transport.delete('parent')
         else:
             self._transport.put_bytes('parent', url + '\n',
-                mode=self.control_files._file_mode)
+                mode=self.bzrdir._get_file_mode())
 
 
 class BzrBranch5(BzrBranch):
@@ -1914,7 +1914,7 @@
         revision_id = _mod_revision.ensure_null(revision_id)
         out_string = '%d %s\n' % (revno, revision_id)
         self._transport.put_bytes('last-revision', out_string,
-            mode=self.control_files._file_mode)
+            mode=self.bzrdir._get_file_mode())
 
     @needs_write_lock
     def set_last_revision_info(self, revno, revision_id):
@@ -2225,7 +2225,7 @@
         # Copying done; now update target format
         new_branch._transport.put_bytes('format',
             format.get_format_string(),
-            mode=new_branch.control_files._file_mode)
+            mode=new_branch.bzrdir._get_file_mode())
 
         # Clean up old files
         new_branch._transport.delete('revision-history')

=== modified file 'bzrlib/counted_lock.py'
--- bzrlib/counted_lock.py	2008-04-24 07:22:53 +0000
+++ bzrlib/counted_lock.py	2008-05-12 06:29:29 +0000
@@ -33,6 +33,9 @@
 
     This can be used with any object that provides a basic Lock interface,
     including LockDirs and OS file locks.
+
+    :ivar _token: While a write lock is held, this is the token 
+        for it.
     """
 
     def __init__(self, real_lock):
@@ -62,17 +65,24 @@
             self._lock_count = 1
             self._lock_mode = 'r'
 
-    def lock_write(self):
+    def lock_write(self, token=None):
         """Acquire the lock in write mode.
 
         If the lock was originally acquired in read mode this will fail.
+
+        :param token: If given and the lock is already held, 
+            then validate that we already hold the real
+            lock with this token.
+
+        :returns: The token from the underlying lock.
         """
         if self._lock_count == 0:
-            self._real_lock.lock_write()
+            self._token = self._real_lock.lock_write(token=token)
             self._lock_mode = 'w'
         elif self._lock_mode != 'w':
             raise ReadOnlyError(self)
         self._lock_count += 1
+        return self._token
 
     def unlock(self):
         if self._lock_count == 0:

=== modified file 'bzrlib/lockable_files.py'
--- bzrlib/lockable_files.py	2008-05-22 05:48:22 +0000
+++ bzrlib/lockable_files.py	2009-03-24 04:49:51 +0000
@@ -23,6 +23,7 @@
 from bzrlib.decorators import (needs_read_lock,
         needs_write_lock)
 import bzrlib.errors as errors
+from bzrlib.counted_lock import CountedLock
 from bzrlib.errors import BzrError
 from bzrlib.osutils import file_iterator, safe_unicode
 from bzrlib.symbol_versioning import (
@@ -63,6 +64,10 @@
     This class is now deprecated; code should move to using the Transport 
     directly for file operations and using the lock or CountedLock for 
     locking.
+    
+    :ivar _lock: The real underlying lock (e.g. a LockDir)
+    :ivar _counted_lock: A lock decorated with a semaphore, so that it 
+        can be re-entered.
     """
 
     # _lock_mode: None, or 'r' or 'w'
@@ -89,6 +94,7 @@
         self._lock = lock_class(transport, esc_name,
                                 file_modebits=self._file_mode,
                                 dir_modebits=self._dir_mode)
+        self._counted_lock = CountedLock(self._lock)
 
     def create_lock(self):
         """Create the lock.
@@ -129,6 +135,9 @@
         
         :deprecated: Replaced by BzrDir._find_modes.
         """
+        # XXX: The properties created by this can be removed or deprecated
+        # once all the _get_text_store methods etc no longer use them.
+        # -- mbp 20080512
         try:
             st = self._transport.stat('.')
         except errors.TransportNotPossible:

=== modified file 'bzrlib/remote.py'
--- bzrlib/remote.py	2008-05-29 20:17:37 +0000
+++ bzrlib/remote.py	2009-03-24 04:49:51 +0000
@@ -1258,12 +1258,12 @@
         self._lock_count = 0
         self._leave_lock = False
 
-    def _ensure_real_transport(self):
+    def _get_real_transport(self):
         # if we try vfs access, return the real branch's vfs transport
         self._ensure_real()
         return self._real_branch._transport
 
-    _transport = property(_ensure_real_transport)
+    _transport = property(_get_real_transport)
 
     def __str__(self):
         return "%s(%s)" % (self.__class__.__name__, self.base)

=== modified file 'bzrlib/repofmt/weaverepo.py'
--- bzrlib/repofmt/weaverepo.py	2008-05-22 05:48:22 +0000
+++ bzrlib/repofmt/weaverepo.py	2009-03-24 04:49:51 +0000
@@ -299,7 +299,8 @@
         try:
             transport.mkdir_multi(['revision-store', 'weaves'],
                 mode=a_bzrdir._get_dir_mode())
-            transport.put_bytes_non_atomic('inventory.weave', empty_weave)
+            transport.put_bytes_non_atomic('inventory.weave', empty_weave,
+                mode=a_bzrdir._get_file_mode())
         finally:
             control_files.unlock()
         return self.open(a_bzrdir, _found=True)

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2008-05-29 12:27:24 +0000
+++ bzrlib/repository.py	2009-03-24 04:49:51 +0000
@@ -1975,7 +1975,7 @@
                 pass
         else:
             self._transport.put_bytes('no-working-trees', '',
-                mode=self.control_files._file_mode)
+                mode=self.bzrdir._get_file_mode())
     
     def make_working_trees(self):
         """Returns the policy for making working trees on new branches."""

=== modified file 'bzrlib/tests/test_counted_lock.py'
--- bzrlib/tests/test_counted_lock.py	2007-05-02 14:00:53 +0000
+++ bzrlib/tests/test_counted_lock.py	2008-05-12 06:29:29 +0000
@@ -39,10 +39,17 @@
         self._lock_mode = 'r'
         self._calls.append('lock_read')
 
-    def lock_write(self):
+    def lock_write(self, token=None):
+        if token is not None:
+            if token == 'token':
+                # already held by this caller
+                return 'token'
+            else:
+                raise errors.TokenMismatch()
         self._assert_not_locked()
         self._lock_mode = 'w'
         self._calls.append('lock_write')
+        return 'token'
 
     def unlock(self):
         self._assert_locked()
@@ -89,7 +96,8 @@
         real_lock.unlock()
         self.assertFalse(real_lock.is_locked())
         # lock write and unlock
-        real_lock.lock_write()
+        result = real_lock.lock_write()
+        self.assertEqual('token', result)
         self.assertTrue(real_lock.is_locked())
         real_lock.unlock()
         self.assertFalse(real_lock.is_locked())
@@ -110,7 +118,7 @@
 
 class TestCountedLock(TestCase):
 
-    def test_lock_unlock(self):
+    def test_read_lock(self):
         # Lock and unlock a counted lock
         real_lock = DummyLock()
         l = CountedLock(real_lock)
@@ -138,7 +146,7 @@
         l = CountedLock(real_lock)
         l.lock_write()
         l.lock_read()
-        l.lock_write()
+        self.assertEquals('token', l.lock_write())
         l.unlock()
         l.unlock()
         l.unlock()
@@ -159,6 +167,34 @@
             ['lock_read', 'unlock'],
             real_lock._calls)
 
+    def test_write_lock_reentrant(self):
+        real_lock = DummyLock()
+        l = CountedLock(real_lock)
+        self.assertEqual('token', l.lock_write())
+        self.assertEqual('token', l.lock_write())
+        l.unlock()
+        l.unlock()
+
+    def test_reenter_with_token(self):
+        real_lock = DummyLock()
+        l1 = CountedLock(real_lock)
+        l2 = CountedLock(real_lock)
+        token = l1.lock_write()
+        self.assertEqual('token', token)
+        # now imagine that we lost that connection, but we still have the
+        # token...
+        del l1
+        # because we can supply the token, we can acquire the lock through
+        # another instance
+        self.assertTrue(real_lock.is_locked())
+        self.assertFalse(l2.is_locked())
+        self.assertEqual(token, l2.lock_write(token=token))
+        self.assertTrue(l2.is_locked())
+        self.assertTrue(real_lock.is_locked())
+        l2.unlock()
+        self.assertFalse(l2.is_locked())
+        self.assertFalse(real_lock.is_locked())
+
     def test_break_lock(self):
         real_lock = DummyLock()
         l = CountedLock(real_lock)

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2008-05-22 05:48:22 +0000
+++ bzrlib/workingtree.py	2009-03-24 04:49:51 +0000
@@ -722,7 +722,7 @@
     def _set_merges_from_parent_ids(self, parent_ids):
         merges = parent_ids[1:]
         self._transport.put_bytes('pending-merges', '\n'.join(merges),
-            mode=self._control_files._file_mode)
+            mode=self.bzrdir._get_file_mode())
 
     @needs_tree_write_lock
     def set_parent_ids(self, revision_ids, allow_leftmost_as_ghost=False):
@@ -805,7 +805,7 @@
         self._must_be_locked()
         my_file = rio_file(stanzas, header)
         self._transport.put_file(filename, my_file,
-            mode=self._control_files._file_mode)
+            mode=self.bzrdir._get_file_mode())
 
     @needs_write_lock # because merge pulls data into the branch.
     def merge_from_branch(self, branch, to_revision=None, from_revision=None,
@@ -1034,7 +1034,7 @@
         self._serialize(self._inventory, sio)
         sio.seek(0)
         self._transport.put_file('inventory', sio,
-            mode=self._control_files._file_mode)
+            mode=self.bzrdir._get_file_mode())
         self._inventory_is_modified = False
 
     def _kind(self, relpath):
@@ -1750,7 +1750,7 @@
         path = self._basis_inventory_name()
         sio = StringIO(xml)
         self._transport.put_file(path, sio,
-            mode=self._control_files._file_mode)
+            mode=self.bzrdir._get_file_mode())
 
     def _create_basis_xml_from_inventory(self, revision_id, inventory):
         """Create the text that will be saved in basis-inventory"""
@@ -2519,7 +2519,7 @@
             return False
         else:
             self._transport.put_bytes('last-revision', revision_id,
-                mode=self._control_files._file_mode)
+                mode=self.bzrdir._get_file_mode())
             return True
 
     @needs_tree_write_lock
@@ -2674,9 +2674,9 @@
         xml5.serializer_v5.write_inventory(inv, sio, working=True)
         sio.seek(0)
         branch._transport.put_file('inventory', sio,
-            mode=branch.control_files._file_mode)
+            mode=branch.bzrdir._get_file_mode())
         branch._transport.put_bytes('pending-merges', '',
-            mode=branch.control_files._file_mode)
+            mode=branch.bzrdir._get_file_mode())
         
 
     def initialize(self, a_bzrdir, revision_id=None, from_branch=None,
@@ -2792,7 +2792,7 @@
         control_files.create_lock()
         control_files.lock_write()
         transport.put_bytes('format', self.get_format_string(),
-            mode=control_files._file_mode)
+            mode=a_bzrdir._get_file_mode())
         if from_branch is not None:
             branch = from_branch
         else:

=== modified file 'bzrlib/workingtree_4.py'
--- bzrlib/workingtree_4.py	2008-05-12 04:07:49 +0000
+++ bzrlib/workingtree_4.py	2008-05-12 06:03:15 +0000
@@ -1309,7 +1309,7 @@
         control_files.create_lock()
         control_files.lock_write()
         transport.put_bytes('format', self.get_format_string(),
-            mode=control_files._file_mode)
+            mode=a_bzrdir._get_file_mode())
         if from_branch is not None:
             branch = from_branch
         else:
@@ -2537,4 +2537,4 @@
         """Change the format marker."""
         tree._transport.put_bytes('format',
             self.target_format.get_format_string(),
-            mode=tree._control_files._file_mode)
+            mode=tree.bzrdir._get_file_mode())

